function [t0,u0] = Evolution(del,sgm)
% The equation parameters
g0 = 0.4; PsatTR = 1; loss = 0.2; Omega = sqrt(10);
gam = 4; beta2 = -2;

% del = 0.05;
% sgm = 0.0065;

% Discretization
N = 512; T = 40; dt = T/N; 
t = (-N/2:1:N/2-1)'*dt;
dw = 2*pi/T; w = [0:N/2 -N/2+1:-1]'*dw;
Psatf = PsatTR/dt*N;

% Operators
L = (1i*beta2*w.^2-loss)/2;
K = (1 - (w/Omega).^2)/2;
g1_fun = @(uf) g0/(1+norm(uf)^2/Psatf);
g2_fun = @(g1,uf) -2*(g1^2/g0/Psatf)*real(dot(uf,(L + g1*K).*uf));
ELOf = @(h,g1,g2) exp(L*h+(g1*h+g2/2*h^2)*K);
u_step13 = @(uf,h,g1,g2) ELOf(h,g1,g2).*uf;

fba = fftshift(exp(-(t/(T/100)).^2));
Nop = @(u) ((1i*gam + del)*abs(u).^2 - sgm*abs(u).^4 - fba).*u;
u_step2 = @(u,h) RK2(h,u,Nop);

% Initial condition
u0 = sech(t);

Z = 300;
h = 1;
uf = fft(u0);

tol_upper = 1e-2;
tol_lower = 1e-5;
z = 0;
z_nextsave = 0;
save_intv = 0.2;
iz = 0;

while z < Z
    uf1 = SSFM_SingleStep(uf,h,g1_fun,g2_fun,u_step13,u_step2);    
    uf0 = SSFM_SingleStep(uf,h/2,g1_fun,g2_fun,u_step13,u_step2);
    uf2 = SSFM_SingleStep(uf0,h/2,g1_fun,g2_fun,u_step13,u_step2);
    
    rel_err = norm(uf1-uf2)/norm(uf1);
%     [h;rel_err]
    if rel_err <= tol_upper
        uf = (4*uf2-uf1)/3;
        z = z + h;
        if rel_err <= tol_lower
            h = h*5/3;
        end
    else
        h = h*2.2/3;
        if h < 1e-6
            error('step size is too small!');
        end
    end
    
    if z >= z_nextsave
        iz = iz + 1;
%         disp([z;h;rel_err]);
        ut = ifft(uf);    
        z_nextsave = z_nextsave + save_intv;        
        usave(:,iz) = ut;
        zsave(iz) = z;          
    end
end

uf = SSFM_SingleStep(uf,Z-z,g1_fun,g2_fun,u_step13,u_step2);
ut = ifft(uf);

% zsave(iz+1) = Z;
% 
% usave(:,iz+1) = ut;
% 
% figure(1);
% mesh(zsave,t,abs(usave(:,1:length(zsave))));
% axis tight;
% view(2);

t0 = t;
u0 = ut;

end

function [ufout,gain] = SSFM_SingleStep(ufin,h,g1_fun,g2_fun,u_step13,u_step2)

g1 = g1_fun(ufin);
g2 = g2_fun(g1,ufin);

u = ifft(u_step13(ufin,h/2,g1,g2));
uf = fft(u_step2(u,h));
g12 = g1_fun(uf);
ufout = u_step13(uf,h/2,g12,g2);

gain = (g1+g12)/2+g2*h;
end

function asol = RK2(h,u,Nop)
u1 = u + h*Nop(u);
asol = u + h/2*(Nop(u) + Nop(u1));
end
