function [t0,u0] = Evolution_convergence()
% The equation parameters
g0 = 0.4; PsatTR = 1; loss = 0.2; Omega = sqrt(10);
gam = 4; beta2 = -2;

del = 0.045;
sgm = 0.0065;

% sgm = 0;

% g0 = 0;
% loss = 0;
gam = 1;
beta2 = -1;
% % del = 0;
% % sgm = 0;

% Discretization
N = 512; T = 50; dt = T/N; 
naxis = (-N/2:1:N/2-1).';
t = dt*naxis;
dw = 2*pi/T; w = fftshift(dw*naxis);
Psatf = PsatTR/dt*N;

% Operators
L = (1i*beta2*w.^2-loss)/2;
K = (1 - (w/Omega).^2)/2;
g1_fun = @(uf) g0/(1+norm(uf)^2/Psatf);
g2_fun = @(g1,uf) -2*(g1^2/g0/Psatf)*real(dot(uf,(L + g1*K).*uf));
% g2_fun = @(g1,uf) 0;
ELOf = @(h,g1,g2) exp(L*h+(g1*h+g2/2*h^2)*K);
u_step13 = @(uf,h,g1,g2) ELOf(h,g1,g2).*uf;

fba = fftshift(exp(-(t/(T/100)).^2));
Nop = @(u) ((1i*gam + del)*abs(u).^2 - sgm*abs(u).^4 - fba).*u;
u_step2 = @(u,h) RK2(h,u,Nop);

% Initial condition
u0 = sech(t);

Z = 10;
hs = 2.^(-10:-6);

for ih = 1:length(hs)
    h = hs(ih);
    Nsteps = round(Z/h);
    uf = fft(u0);
    for istep = 1:Nsteps
        uf1 = SSFM_SingleStep(uf,h,g1_fun,g2_fun,u_step13,u_step2);
%         uf = uf1;
        
        uf0 = SSFM_SingleStep(uf,h/2,g1_fun,g2_fun,u_step13,u_step2);
        uf2 = SSFM_SingleStep(uf0,h/2,g1_fun,g2_fun,u_step13,u_step2);        
        uf = (4*uf2-uf1)/3;        
    end
    if ih == 1
        uref = uf;
    else
        err(ih-1) = norm(uref-uf);
    end
end

save temp;

figure(1);
plot(log2(hs(2:end)),log2(err),'-+');
axis tight;
grid on;


end

function [ufout,gain] = SSFM_SingleStep(ufin,h,g1_fun,g2_fun,u_step13,u_step2)

g1 = g1_fun(ufin);
g2 = g2_fun(g1,ufin);

u = ifft(u_step13(ufin,h/2,g1,g2));
uf = fft(u_step2(u,h));

g12 = g1_fun(uf);
ufout = u_step13(uf,h/2,g12,g2);

gain = (g1+g12)/2+g2*h;
end

function asol = RK2(h,u,Nop)
u1 = u + h*Nop(u);
asol = u + h/2*(Nop(u) + Nop(u1));
end
