function bound_las = find_amplitude_boundary_LAS()
% clc;
format long;
% close all;

tic

del = 0.031;
sgm = 0;

a = hme_eigen();
IS.T = 50*a.tau0;
IS.N = 512;
IS.Uout = zeros(IS.N,1);
naxis = (-IS.N/2:IS.N/2-1).';
IS.t = IS.T/IS.N*naxis;

a.setup(IS,del,sgm) ;

TS.mode = 'tw2tau';
TS.N = 512;
TS.tw2tau = 50;

TS.switch_FourierFD = 'off';

IS.Uout = a.A0*sech(a.t/a.tau0).^(1+1i*a.beta0);
IS.phi0 = a.phi0;

a.newton_solver(IS.Uout,IS.phi0,TS);

coord = [sgm;del];
Out_U = a.NTout;

del_jump = 0.00005; 
sgm_jump = 0.001;

res_thrsh = 1e-9;
ew_lower_thr = 1e-5;  % the lower limit for evaluating the amplitude eigenvalue
ew_upper_thr = 3e-2;  % the upper limit for the amplitude eigenvalue to be useful for evaluating the boundary

del = del  + del_jump;
% record the stability boundaries for the amplitude mode instability of the low amplitude solution
bound_las = []; 
ew_qual = [];  % to record "qualified" eigenvalues within this column
del_qual = [];  % to record "qualified" dels within this column
del_old = del;  % to record the value of the last del that leads to a converging result
ew_amp = [];  % to record the amp eigenvalues with the same sigma
count_same_sgm = 0;  % record the number of calculated points with the same sigma

%% Calculating the boundary with the first value of sigma
while (length(bound_las) < 3)
    idx = find_initial(coord,[sgm,del]);
    fprintf('Computing [%.7g %.7g] from [%.7g %.7g]\n', del, sgm, coord(2,idx), coord(1,idx));
    if del < 0
        error('Negative delta!')
    end
    IS = Out_U(idx);    
    a = hme_eigen();
    a.setup(IS,del,sgm) ;
    a.newton_solver(IS.Uout,IS.phi,TS);       
   
    % If Newton solver is not converging, we move to a "closer" point to
    % the parameters of known cases
    if a.NTout.res > res_thrsh
        del_jump = del_jump/17*15;
        del = del_old + del_jump;
        disp('Case 1. Not converging');
        continue;
    end
    
    % If Newton solver converges, we record the current result to our
    % database and continue to calculate the linearized spectrum
    coord = [coord,[sgm;del]];
    Out_U = [Out_U,a.NTout];
    del_old = del;
    a.CompleteEigen('on');
    ewc = find_amp_mode(a.ew,a.ev,ew_lower_thr);
    ew_amp = [ew_amp,ewc]; 
    count_same_sgm = count_same_sgm + 1; % to record number of points with the current value of sigma
    
    % this condition module controls data-saving of this column
    if abs(ewc) <= ew_upper_thr
        fprintf('This amplitude eigenvalue is good!\n');
        ew_qual = [ew_qual ewc];    % to record "qualified" eigenvalues that are close enough to 0
        del_qual = [del_qual a.del];  % to record values of delta of qualified points
        
        [~,ind] = sort(abs(ew_qual),'ascend');
        ew_qual = ew_qual(ind);
        del_qual = del_qual(ind);        
        del = del + del_jump;
    elseif count_same_sgm == 2  % Only two points in this column but not close enough to 0
        fprintf('Two points in the column, and not close enough to 0!\n');
        del = del + del_jump;
    elseif count_same_sgm >= 3
        fprintf('Three points in the column, but not close enough to 0!\n');
        del = find_root(ew_amp(end-2:end),coord(2,end-2:end));
    else  % There is only one point in this column. We continue to find the next point.
        fprintf('Only one point in the column, and not close enough to 0!\n');
        del = del + del_jump;
    end
    
    % this condition finish locating and calculating the boundary using the three points 
    if length(ew_qual) == 3
        bound = find_root(ew_qual,del_qual);
        bound_las = [bound_las [sgm;bound]];
        sgm_old = sgm;
        sgm = sgm + sgm_jump;
        count_same_sgm = 0;
        dels_old = del_qual;        
        ew_qual = [];        
        del_qual = [];
        fprintf('Now we compute a new ''sgm''!!\n');
    end        
end

% save temp0;

% load temp0;

%% The loop of boundary tracking when sigma increases
sgm_end = 0.021;
next_sigma_flag = 1;
slope_bound = 0;  % d(boundary)/d(sigma)
while next_sigma_flag
    i_del = 1;
    ew_qual = [];
    del_qual = [];
    ew_amp = [];
    while i_del < 4  % to finish one column of dels
        if del < 0
            error('Negative delta!')
        end
        idx = find_initial(coord,[sgm,del]);       
        fprintf('Computing [%.7g %.7g] from [%.7g %.7g]\n', del, sgm, coord(2,idx), coord(1,idx));
        IS = Out_U(idx);
        a = hme_eigen();
        a.setup(IS,del,sgm) ;
        a.newton_solver(IS.Uout,IS.phi,TS);    
        
        if a.NTout.res > res_thrsh
            disp('Case 1. Not converging');
            if i_del == 1   % if this is the first point in the current column, reset sigma
                sgm_jump = sgm_jump * 0.7;
                sgm = sgm_old + sgm_jump;
            else   % if this is NOT the first point in the current column, reset delta
                del_jump = del_jump/3;
                del = del_old + del_jump;
                continue;
            end
        end
        
        % If Newton solver converges, we record the current result to our
        % database and continue to calculate the linearized spectrum
        coord = [coord,[sgm;del]];
        Out_U = [Out_U,a.NTout];
        del_old = del;        
        a.CompleteEigen('on');
        ewc = find_amp_mode(a.ew,a.ev,ew_lower_thr);
        ew_amp = [ew_amp,ewc];
        count_same_sgm = count_same_sgm + 1;                       
        
        % this condition module controls data-saving
        if abs(ewc) <= ew_upper_thr
            fprintf('This amplitude eigenvalue is good!\n');
            ew_qual = [ew_qual ewc];
            del_qual = [del_qual a.del];            
            [~,ind] = sort(abs(ew_qual),'ascend');
            ew_qual = ew_qual(ind);
            del_qual = del_qual(ind);
            del = del + del_jump;
            i_del = i_del + 1;            
        elseif count_same_sgm == 2
            fprintf('Two points in the column, and not close enough to 0!\n');
            del = del + del_jump;
        elseif count_same_sgm >= 3
            fprintf('Three points in the column, but not close enough to 0!\n');
            del = find_root(ew_amp(end-2:end),coord(2,end-2:end));
        else  % There is only one point in this column. We continue to find the next point.
            fprintf('Only one point in the column, and not close enough to 0!\n');
            del = del + del_jump;
            disp(del_jump);
        end                       
        
        % this condition finish locating and compute the three points for interpolation
        if length(ew_qual) == 3
            if monotonicity(ew_qual,del_qual) 
                bound = find_root(ew_qual,del_qual);
                bound_las = [bound_las [sgm;bound]];
                slope_bound = (bound_las(2,end)-bound_las(2,end-1))/(bound_las(1,end)-bound_las(1,end-1));
                sgm_old = sgm;
                sgm = sgm + sgm_jump;
                if sgm >= sgm_end
                    next_sigma_flag = 0;
                else
                    count_same_sgm = 0;
                    dels_old = del_qual;
                    del = dels_old(1) + sgm_jump*slope_bound;
                    fprintf('Now we compute a new ''sgm''!!\n');
                end
            else
                next_sigma_flag = 0;
            end
        end
    end
    
    plot(bound_las(1,:),bound_las(2,:));
    xlim([0 sgm_end]);
    ylim([0 0.06]);
    xlabel('$\sigma$','interpreter','latex');
    ylabel('$\delta$','interpreter','latex');
    drawnow
end 
toc
% save temp.mat;

% figure(1);
% plot(bound_las(1,:),bound_las(2,:));
% grid on;
% axis tight;
% xlabel('$\sigma$','interpreter','latex');
% ylabel('$\delta$','interpreter','latex');

end