classdef hme_eigen < hme_newton
    
    properties        
        ev
        ew
        J0
    end
    
    methods
        function obj=hme_eigen()
        end
        
        function D = PartEigen(self,ew0,k,switch_FourierFD) 
            self.Jacobian(switch_FourierFD,[real(self.NTout.Uout);imag(self.NTout.Uout);self.NTout.phi]);                            
            [self.ev, D] = eigs(self.J0,k,ew0);
            self.ew = diag(D);
            D = self.ew;
        end                
        
        function D = CompleteEigen(self,switch_FourierFD) 
            self.Jacobian(switch_FourierFD,[real(self.NTout.Uout);imag(self.NTout.Uout);self.NTout.phi]);   
            
%             f = @(x0) self.Jacobian2(x0,switch_FourierFD,self.NTout.phi);
%             for ip = 2:5
%                 pw = 10^(-ip);
%                 figure(67);
%                 x0 = [real(self.NTout.Uout);imag(self.NTout.Uout)];
%                 Dx = x0*pw;
%                 [F,J] = f(x0);
%                 Df = abs(f(x0+Dx) - F - J*Dx);
%                 plot(log10(Df)); hold on; grid on;
%             end
%             
            [X, D] = eig(full(self.J0));
            D = diag(D);
            [self.ev, D] = self.sort_EigenOutput_complex(X,D);
            self.ew = D;
        end                
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% Some Utilities
    methods    
        function [F,J0] = Jacobian (self,switch_FourierFD,x) 
            if strcmp(switch_FourierFD,'on')
                [~, self.Df2] = fourdif(self.N,2);
                self.Df2 = self.dw^2*self.Df2;
                self.dispD2 = self.b2*self.Df2;       % dispersion operator
                self.gainD2 = (self.I + (1/self.Omega^2)*self.Df2);  %  Gain multiplier
            end
            
            vi = x(1:self.N);
            wi = x(self.N+1:self.N*2);
            phi = x(end);
            Pi = norm(vi+1i*wi)^2;
            gi = self.g0/(1 + Pi/self.Psat);
            vi2 = vi.^2;
            wi2 = wi.^2;
            ui2 = vi2 + wi2;
            ui4 = ui2.^2;
            viwi = vi.*wi;                       
                        
            F1v = gi*self.gainD2/2*(self.I - 2/(self.Psat + Pi)*vi*(vi)') - spdiags(self.l/2 ...
                + 2*self.gam*viwi - self.del*(3*vi2+wi2) + self.sgm*(ui4 + 4*vi2.*ui2), 0,self.N,self.N);
            F1w = self.dispD2/2 - gi^2/(self.g0*self.Psat)*self.gainD2*vi*(wi)' + spdiags(phi ...
                - self.gam*(vi2+3*wi2) + 2*self.del*viwi - 4*self.sgm*viwi.*ui2, 0,self.N,self.N);
            F2v = -self.dispD2/2 - gi^2/(self.g0*self.Psat)*self.gainD2*wi*(vi)' + spdiags(-phi...
                + self.gam*(3*vi2+wi2) + 2*self.del*viwi - 4*self.sgm*viwi.*ui2, 0,self.N,self.N);
            F2w = gi*self.gainD2/2*(self.I - 2/(self.Psat + Pi)*wi*(wi)') + spdiags(-self.l/2 ...
                + 2*self.gam*viwi + self.del*(vi2+3*wi2) - self.sgm*(ui4 + 4*wi2.*ui2),0,self.N,self.N);
            J0 = [F1v, F1w; F2v, F2w];
            self.J0 = J0;
           
        end
        
        function [F,J] = Jacobian2 (self,x,switch_FourierFD,phi) 
            if strcmp(switch_FourierFD,'on')
                [~, self.Df2] = fourdif(self.N,2);
                self.Df2 = self.dw^2*self.Df2;
                self.dispD2 = self.b2*self.Df2;       % dispersion operator
                self.gainD2 = (self.I + (1/self.Omega^2)*self.Df2);  %  Gain multiplier
            end
            
            vi = x(1:self.N);
            wi = x(self.N+1:self.N*2);
            Pi = norm(vi+1i*wi)^2;
            gi = self.g0/(1 + Pi/self.Psat);
            vi2 = vi.^2;
            wi2 = wi.^2;
            ui2 = vi2 + wi2;
            ui4 = ui2.^2;
            viwi = vi.*wi;
            
            if 0
                vd2 = (self.Df2_fun(vi));
                wd2 = (self.Df2_fun(wi));
            else
                vd2 = self.Df2*vi;
                wd2 = self.Df2*wi;
            end
            
            F1 = (gi-self.l).*vi + 2*phi*wi + self.b2*wd2 + gi/self.Omega^2*vd2 - 2*self.gam*ui2.*wi...
                + 2*(self.del*ui2 - self.sgm*ui4).*vi;                                    
            F1v = gi*self.gainD2/2*(self.I - 2/(self.Psat + Pi)*vi*(vi)') - spdiags(self.l/2 ...
                + 2*self.gam*viwi - self.del*(3*vi2+wi2) + self.sgm*(ui4 + 4*vi2.*ui2), 0,self.N,self.N);
            F1w = self.dispD2/2 - gi^2/(self.g0*self.Psat)*self.gainD2*vi*(wi)' + spdiags(phi ...
                - self.gam*(vi2+3*wi2) + 2*self.del*viwi - 4*self.sgm*viwi.*ui2, 0,self.N,self.N);
            
%             F = [F1]/2;
%             J = [F1v, F1w];
            
            F2 = (gi-self.l).*wi - 2*phi*vi - self.b2*vd2 + gi/self.Omega^2*wd2 + 2*self.gam*ui2.*vi...
                + 2*(self.del*ui2 - self.sgm*ui4).*wi;
            F2v = -self.dispD2/2 - gi^2/(self.g0*self.Psat)*self.gainD2*wi*(vi)' + spdiags(-phi...
                + self.gam*(3*vi2+wi2) + 2*self.del*viwi - 4*self.sgm*viwi.*ui2, 0,self.N,self.N);
            F2w = gi*self.gainD2/2*(self.I - 2/(self.Psat + Pi)*wi*(wi)') + spdiags(-self.l/2 ...
                + 2*self.gam*viwi + self.del*(vi2+3*wi2) - self.sgm*(ui4 + 4*wi2.*ui2),0,self.N,self.N);
%             
            F = [F1;F2]/2;
            J = [F1v, F1w; F2v, F2w];
        end
                          
        function out = check_symm(self,u)
            uleft = u(1:self.N-1);
            uright = flipud(uleft);
            addition = norm(uleft + uright);
            subtraction = norm(uleft - uright);
            if (addition < subtraction)
                out = 'odd ';
            else
                out = 'even';
            end
        end
    end
    
    methods (Static)
        function [V_out, D_out] = sort_EigenOutput_complex(V,D)
            [~,orders] = sort(abs(imag(D)),'ascend');
            D_tmp = D(orders);
            if length(D) > 3  % if there are four discrete eigenvalues
                Dtmp2 = D_tmp(1:4);
                [~, order_d] = sort(abs(real(D_tmp(1:4))),'ascend');
                D_out = [Dtmp2(order_d); D_tmp(5:end)];
                V_out = V(:,orders);
                V_out(:,1:4) = V_out(:,order_d);
            else
                [D_out order_d] = sort(real(D_tmp),'descend');
                V_out = V(:,order_d);
            end
        end                
    end
    
end
