function bound_cont = find_continuum_boundary()
% clc;
format long;
% close all;

tic

del = 0.011;
sgm = 0;

a = hme_newton();
IS.T = 50*a.tau0;
IS.N = 512;
IS.Uout = zeros(IS.N,1);
naxis = (-IS.N/2:IS.N/2-1).';
IS.t = IS.T/IS.N*naxis;

a.setup(IS,del,sgm) ;

TS.mode = 'tw2tau';
TS.N = 512;
TS.tw2tau = 50;

TS.switch_FourierFD = 'off';

IS.Uout = a.A0*sech(a.t/a.tau0).^(1+1i*a.beta0);
IS.phi0 = a.phi0;

a.newton_solver(IS.Uout,IS.phi0,TS);

coord = [sgm;del];
Out_U = a.NTout;

del_jump = 0.001;
sgm_jump = 0.005;
ew = [];
dd = [];
bound = [];
bound_flag = 0;
res_thrsh = 1e-9;
gl_thrsh = 0.003;

del = del  + del_jump;
bound_cont = [];
gl_col = [];
del_col = [];
g_l = [];
count_same_sgm = 0;

while (length(bound_cont) < 3)
    idx = find_initial(coord,[sgm,del]);
    fprintf('Computing [%.7g %.7g] from [%.7g %.7g]\n', del, sgm, coord(2,idx), coord(1,idx));
    IS = Out_U(idx);    
    a = hme_newton();
    a.setup(IS,del,sgm) ;
    a.newton_solver(IS.Uout,IS.phi,TS);        
   
    % database for initial values
    if a.NTout.res > res_thrsh
        del_jump = del_jump/3;
        del = del - del_jump;
        disp('Case 1. Not converging');
        continue;
    end
    
    coord = [coord,[sgm;del]];
    Out_U = [Out_U,a.NTout];
    g_l = [g_l,a.NTout.g_l];
    count_same_sgm = count_same_sgm + 1;
    
    % this condition module controls data-saving of this column
    if abs(a.NTout.g_l) <= gl_thrsh
        fprintf('g-l is small enough!\n');
        gl_col = [gl_col a.NTout.g_l];
        del_col = [del_col del];
        
        [~,ind] = sort(abs(gl_col),'ascend');
        gl_col = gl_col(ind);
        del_col = del_col(ind);        
        del = del + del_jump;
    elseif count_same_sgm == 2
        del = find_root(g_l(end-1:end),coord(2,end-1:end));
    elseif count_same_sgm >= 3
        del = find_root(g_l(end-2:end),coord(2,end-2:end));        
    else
        del = del + del_jump;
    end
    

    % this condition finish locating and compute the three points for
    % interpolation
    if length(gl_col) == 3
        if max(gl_col)*min(gl_col) > 0 % in case the points are all stable/unstable
            del = find_root(gl_col,del_col) - del_jump*rand/7; % a little deviation to the other sign
            gl_col = gl_col(1:2);
            del_col = del_col(1:2);    
            fprintf('Find the third del!!\n');
        else
            bound = find_root(gl_col,del_col);
            bound_cont = [bound_cont [sgm;bound]];
            sgm = sgm + sgm_jump;
            count_same_sgm = 0;
            gl_col = [];
            dels_old = del_col;
            del_col = [];     
            fprintf('Now we compute a new ''sgm''!!\n');
        end
    end
        
end

% save('temp1.mat','-v7.3')
% 
% load temp1.mat;

sgm_end = 0.02;

while sgm <= sgm_end   
    i_del = 1;
    slope_bound = (bound_cont(2,end)-bound_cont(2,end-1))/(bound_cont(1,end)-bound_cont(1,end-1));
    
    while i_del < 3  % to finish one column of dels       
        
        idx = find_initial(coord,[sgm,del]);
        fprintf('Computing [%.7g %.7g] from [%.7g %.7g]\n', del, sgm, coord(2,idx), coord(1,idx));
        IS = Out_U(idx);
        a = hme_newton();
        a.setup(IS,del,sgm) ;
        a.newton_solver(IS.Uout,IS.phi,TS);
        
        if a.NTout.res > res_thrsh
            del_jump = del_jump/3;
            del = del - del_jump;
            disp('Case 1. Not converging');
            continue;
        end
        
        coord = [coord,[sgm;del]];
        Out_U = [Out_U,a.NTout];
        g_l = [g_l,a.NTout.g_l];
        count_same_sgm = count_same_sgm + 1;
        
        % this condition module controls data-saving
        if abs(a.NTout.g_l) <= gl_thrsh
            fprintf('g-l is small enough!\n');
            gl_col = [gl_col a.NTout.g_l];
            del_col = [del_col del];
            
            [~,ind] = sort(abs(gl_col),'ascend');
            gl_col = gl_col(ind);
            del_col = del_col(ind);
            i_del = i_del + 1;
            del = dels_old(i_del) + sgm_jump*slope_bound;
        elseif count_same_sgm == 2
            del = find_root(g_l(end-1:end),coord(2,end-1:end));            
        elseif count_same_sgm >= 3
            del = find_root(g_l(end-2:end),coord(2,end-2:end));            
        end
        
        % this condition finish locating and compute the three points for
        % interpolation
        if length(gl_col) == 3
            if max(gl_col)*min(gl_col) > 0 % in case the points are all stable/unstable
                del = find_root(gl_col,del_col) - del_jump*rand/7; % a little deviation to the other sign
                gl_col = gl_col(1:2);
                del_col = del_col(1:2);
                i_del = 2;                
                fprintf('Find the third del!!\n');
            else
                bound = find_root(gl_col,del_col);
                bound_cont = [bound_cont [sgm;bound]];
                sgm = sgm + sgm_jump;
                count_same_sgm = 0;
                gl_col = [];
                dels_old = del_col;
                del_col = [];
                del = dels_old(1) + sgm_jump*slope_bound;
                fprintf('Now we compute a new ''sgm''!!\n');
            end
        end
    end
    
    plot(bound_cont(1,:),bound_cont(2,:));
    xlim([0 sgm_end]);
    ylim([0 0.06]);
    xlabel('$\sigma$','interpreter','latex');
    ylabel('$\delta$','interpreter','latex');
    drawnow
end 
toc
% save temp.mat;
% 
% figure(1);
% plot(bound_cont(1,:),bound_cont(2,:));
% grid on;
% axis tight;
% xlabel('$\sigma$','interpreter','latex');
% ylabel('$\delta$','interpreter','latex');

end